"3.1.1.0"
!  

CPScrollingPane subclass: #CPColumnarListBox
  instanceVariableNames: 
    ' objects fields showHeader totalFieldWidth headerHeight showLines colorMethod disableMethod selections '
  classVariableNames: ''
  poolDictionaries: 
    ' CharacterConstants CursorConstants OperatingSystemConstants VirtualKeyConstants '!

!CPColumnarListBox methods !
button1DownMultiSelect: aPoint
    "Private"

    | cell point currentSelection lastSelection |
    (self subpaneButton1Down: aPoint)
        ifFalse: [^self].
    self doSetFocus.
    currentSelection := selectedCell.
    cell := self boundedCellForPoint: self mouseLocation.
    (self inBounds: cell)
        ifFalse: [^nil].
    (self isSelected: cell y)
        ifTrue:
            [self rawSelections remove: cell y ifAbsent: [nil].
            self unHighlightCell: cell.
            currentSelection := selectedCell := cell.
            self showFocusInCell: cell.
            self triggerMultiSelectEvent]
        ifFalse:
            [self isOkToChange == false
                ifFalse:
                    [(self isDisabled: cell)
                        ifFalse:
                            [currentSelection := selectedCell := cell.
                            self rawSelections add: cell y.
                            self highlightCell: cell.
							self triggerMultiSelectEvent.
                            ]
                        ifTrue:
                            [self hideFocus.
                            ].
                    ].
            ].
    lastSelection := currentSelection.
    self
        whileMouseStillDown: [:currentMousePosition |
            point := self mouseLocation.
            cell := self boundedCellForPoint: point.
            (self cell: currentSelection differsFrom: cell) & (self inBounds: cell)
                ifTrue:
                    [self hideFocus.
                    lastSelection := currentSelection.
                    self dragScrolls
                        ifTrue:
                            [self keepCellVisible: cell.
                            ].
                    self showFocusInCell: cell.
                    selectedCell := currentSelection := cell.
                    ].
            ].
    selectedCell := currentSelection.
    (self isDisabled: currentSelection)
        ifTrue:
            [self hideFocus.
            selectedCell := lastSelection.
            self showFocusInCell: lastSelection.
            (Smalltalk at: #Terminal) bell. "UserLibrary messageBeep: 0" ].! !

!CPColumnarListBox methods !
button2Up: aPoint
    "Private - Right button was clicked.
    If column sorting is on and aPoint is in a column header, sort the rows based on the column contents
    (descending sort if the shift key is down; ascending otherwise)."

    (self canSort and: [self isPointInHeader: aPoint])
        ifTrue: [
			self sortOn: self columnClicked ascending: Notifier isShiftKeyDown not]
		ifFalse: [
			super button2Up: aPoint]! !

!CPColumnarListBox methods !   
defaultDroppedOn: aDragDropList
    "Accept the drag and drop."

    | addList index collection |
    #obsoleteMethod.
    addList := OrderedCollection new.
    aDragDropList items
        do:
            [:each |
            ((self contents includes: each) not and: [self isCompatible: each])
                ifTrue: [addList add: each].
            ].
    addList isEmpty
        ifTrue: [^self].
    index := ((self cellForPoint: aDragDropList location) y min: objects size + 1) max: 1.
    collection := self contents asOrderedCollection.
    addList
        reverseDo:
            [:each |
            collection add: each beforeIndex: index].
    self contents: collection asOrderedCollection.! !

!CPColumnarListBox methods !
restoreSelected: anObject
    "Private"

    | topIndex |
    self disableRedraw.
    topIndex := self getTopIndex.
	self triggerNeedsContentsEvent.
    self setTopIndex: topIndex.
    self selection: anObject.
    self enableRedraw.
    self invalidateRect: self rectangle! !

!CPColumnarListBox methods !
isCompatibleObject: anObject
	"Is <anObject> compatible with the receiver"

	| result |
	(result := self dragCompability) notNil ifTrue: [
		result := result evaluateWithArguments: (Array with: anObject).
		result == true ifTrue: [^true].
		result == false ifTrue: [^false]].
    self columnHeaders do: [ :eachColumn |
    	(anObject respondsTo: eachColumn selector) ifFalse: [
    		^false ] ].
    ^true
! !

!CPColumnarListBox methods ! 
searchNameForIndex: anIndex
	"Private"

	| name |
	fields isEmpty | anIndex isNil
		ifTrue: [^nil].
	name := (objects at: anIndex) wbAttributeNamed: (fields at: 1) selector.
	name isBitmap
		ifTrue: [^nil].
	name isString not
		ifTrue:
			[name := name printString.
			].
	^name! !

!CPColumnarListBox methods !
resizeField: field
start: aPoint
    "Private"

    | currentPoint lastPoint tooThin offset cursorLocation minimumFieldWidth maximumFieldWidth maxCurrentPointX |
    currentPoint := aPoint.
    minimumFieldWidth := self minimumFieldWidth.
    tooThin := field fieldPixelWidth < minimumFieldWidth.
    maximumFieldWidth := self width - 25.
    maxCurrentPointX := (currentPoint x right: maximumFieldWidth) left: field fieldPixelWidth.
    self
        whileMouseStillDownPrim: [:currentMousePosition |
            cursorLocation := self mouseLocation.
            offset := cursorLocation x - currentPoint x.
            (offset ~~ 0 and: [(offset < 0 & tooThin) not])
                ifTrue:
                    [(field fieldPixelWidth + offset) < minimumFieldWidth
                        ifTrue:
                            [currentPoint := (currentPoint x - (field fieldPixelWidth - minimumFieldWidth)) @ cursorLocation y.
                            field fieldPixelWidth: minimumFieldWidth.
                            tooThin := true.
                            ]
                        ifFalse:
                            [currentPoint := (cursorLocation x leftMost: maxCurrentPointX) @ cursorLocation y.
                            field fieldPixelWidth: (field fieldPixelWidth + offset min: maximumFieldWidth).
                            tooThin := false.
                            ].
                    self displayHeader.
                    lastPoint isNil
                        ifFalse: [self drawMovingLine: lastPoint x].
                    self drawMovingLine: currentPoint x.
                    lastPoint := currentPoint.
                    ].
            ].
    self updateTotalFieldWidth.
    self redraw.
    self updateRectangle.
    self updateHorizontalSlider.
	self triggerColumnResizedEvent.! !

!CPColumnarListBox methods !  
restore
    "Private - Refresh the list from the owner
      and maintain the position in the list
      without selecting it."

    | first |
    first := self getTopIndex.
	self triggerNeedsContentsEvent.
    self setTopIndex: first! !

!CPColumnarListBox methods ! 
cell: cell1
differsFrom: cell2
    "Private - answer whether one cell differs from another"

    cell1 isNil
        ifTrue: [^cell2 notNil].
    ^cell1 y ~= cell2 y! !

!CPColumnarListBox methods !
sortOn: column
ascending: aBoolean
    "Sort the matrix on the specified column."

    | selector av bv |
    column == 0 ifTrue: [^self].
    selector := (fields at: column) selector.
    self
        contentsNoResize:
            (aBoolean
					ifTrue:
						[(self contents
							asSortedCollection:
								[:a :b |
								av := a wbAttributeNamed: selector.
								bv := b wbAttributeNamed: selector.
								[av <= bv] on: Error do: [av asString <= bv asString]])]
					ifFalse:
						[(self contents
							asSortedCollection:
								[:a :b |
								av := a wbAttributeNamed: selector.
								bv := b wbAttributeNamed: selector.
								[av >= bv] on: Error do: [av asString >= bv asString]])]) asOrderedCollection! !

!CPColumnarListBox methods ! 
defaultColorMethod
    "Private"

    ^nil! !

!CPColumnarListBox class methods !
constructEventsTriggered
    "Private - answer the set of events that instances of the
     receiver can trigger."

    ^super
        constructEventsTriggered addAll: #(#getFields #columnResized: #columnReordered: #multipleSelect:);
        yourself! !

!CPColumnarListBox methods !   
restoreSelected
    "Refresh the list from the owner
     and keep the old selection. "

    | index first |
    index := self selections.
    first := self getTopIndex.
    self disableRedraw.
	self triggerNeedsContentsEvent.
    self setTopIndex: first.
    self selections: index.
    self enableRedraw.
    self invalidateRect: self rectangle! !

!CPColumnarListBox methods ! 
isMultiSelect
    "Answer whether the receiver is in multi selection mode."

    ^self cpStyle == #multipleSelect! !

!CPColumnarListBox methods !   
button1DownShift: aPoint
    "Private"

    (self isPointInHeader: aPoint) ifFalse: [
        self isExtendedSelect ifTrue: [
            self doGraphics: [
                self button1DownExtendedSelect: aPoint shift: true control: Notifier isControlKeyDown].
            ^self]].
    ^super button1DownShift: aPoint! !

!CPColumnarListBox methods !  
selection: anItem
    "Set the selection of the listbox to anItem"

    | oldCell |
    anItem isNil
        ifTrue:
            [selectedCell notNil & self isValid
                ifTrue:
                    [self invalidateRect: (self rectForCell: selectedCell).
                    self updateRectangle.
                    ].
            ^selectedCell := nil].
    anItem isInteger
        ifTrue:
            [oldCell := selectedCell.
            selectedCell := 1 @ ((anItem max: 0) min: objects size).
            oldCell = selectedCell
                ifTrue: [^self].
            self doGraphics: [
                self isValid
                    ifTrue:
                        [oldCell notNil
                            ifTrue:
                                [self unHighlightCell: oldCell.
                                ].
                        selectedCell notNil
                            ifTrue:
                                [self highlightCell: selectedCell.
                                ].
                        self keepSelectionVisible.
                        ]].
            ]
        ifFalse:
            [self selection: (objects indexOf: anItem).
            ].! !

!CPColumnarListBox methods !   
calcFieldSizes
	"Private - calculate new field sizes"

	| extent |
	objects isNil | fields isNil ifTrue: [^self].
	totalFieldWidth := 0.
	headerHeight := 0.
	fields do: [:f |
		self wbDisplayContext font: self font.
		extent := f wbDisplayExtentWith: self wbDisplayContext.
		headerHeight := headerHeight max: extent y.
		extent := extent x @ (f font ifNil: [self font]) height.
		f autoFieldWidth
			ifTrue: [
				1 to: (objects size min: self class maxRowsToAutoSize) do: [:i |
					f selector notNil ifTrue: [
						extent := extent max: (
							f
								wbDisplayExtent: ((objects at: i) wbAttributeNamed: f selector)
								with: self wbDisplayContext)]].
				f fieldPixelWidth: extent x]
			ifFalse: [
				"reset the cached width for fixed width fields"
				f calculateCachedWidth].
		cellSize y: (cellSize y max: extent y + 2).
		totalFieldWidth := totalFieldWidth + self fieldMargin + f fieldPixelWidth].
	totalFieldWidth := totalFieldWidth + self fieldMargin.
	totalCells := ((totalFieldWidth - 1) // self cellSize x + 1) @ objects size.
	headerHeight := headerHeight + self headerMargin.! !

!CPColumnarListBox methods !   
rawSelections: aCollection
    "Private"

    selections := aCollection! !

!CPColumnarListBox methods ! 
fullExtent
    "Private - answer the total extent of the objects + the header"

    ^(totalFieldWidth @ (self totalCells y * self cellSize y)) + self headerExtent.! !

!CPColumnarListBox methods ! 
fields
    "Answer the fields for this listbox"

    ^fields! !

!CPColumnarListBox methods !
dragTargetEraseItemEmphasis
        "Private - erase the previously drawn item emphasis."
    | lastIndex rect |
    lastIndex := self dragTargetSelection.
    lastIndex isNil ifTrue: [^self].
    rect := self rectForCell: 1 @ lastIndex.
    self dragTargetDrawEmphasisRectangle: rect! !

!CPColumnarListBox methods ! 
defaultShowHeader
    "Private"

    ^true! !

!CPColumnarListBox methods !  
displayObjects
    "Private - loop through the objects, displaying each one in turn"

    | range startPos |
    range := self cellsToDraw.
    startPos := self pointForCell: 1 @ range top.
    self pen backColor: self backColor.
    (range top max: 1) to: (range bottom min: objects size)
        do:
            [:i |
            self
                    displayItem: (objects at: i)
                    inRect: (Rectangle leftTop: startPos extent: self scrollAreaRectangle width @ self cellSize y)
                    highlight: false.
            startPos y: (startPos y down: self cellSize y).
            ].! !

!CPColumnarListBox class methods ! 
supportedEvents
    "Private"

    ^super
        supportedEvents add: #getFields;
        add: #columnResized;
        add: #columnReordered;
        yourself! !

!CPColumnarListBox methods ! 
dragTargetDrawItemEmphasis: aPoint
        "Private - draw the target emphasis for the receiver
        given the cursor location aPoint (relative to the receiver).
        Emphasis is drawn indicating individual list items as
        individual targets."

    | index lastIndex rect |
    index := (self boundedCellForPoint: aPoint) y.
        "only update if changed"
    lastIndex := self dragTargetSelection.
    index = lastIndex ifTrue: [ ^nil ].  "don't draw emphasis if already drawn."
    lastIndex notNil ifTrue: [ self dragTargetEraseItemEmphasis ].
    rect := self rectForCell: 1 @ index.
    self dragTargetDrawEmphasisRectangle: rect.
    self dragTargetSelection: index.
! !

!CPColumnarListBox methods ! 
contents: objectList
fields: aFieldList
    "Set the contents and fields of the listbox"

    objects := objectList.
    fields := aFieldList.
    self calcFieldSizes.
    selectedCell := nil.
    self isMultiSelect | self isExtendedSelect
        ifTrue: [selections := Set new].
    self isValid
        ifTrue:
            [self adjustAutoFields.
			self invalidateRect: nil.
            self updateRectangle.
            ].! !

!CPColumnarListBox methods ! 
extendedSelect
    "Set the listbox to extended select mode"

    self rawSelections: Set new.
    self propertyAt: #cpStyle put: #extendedSelect.! !

!CPColumnarListBox methods ! 
cursorOnFieldBoundary: aPoint
    "Private"

    ^(self fieldToResizeForPoint: aPoint) notNil! !

!CPColumnarListBox methods !   
selectedItems
    "Answer the selected items"

    self isMultiSelect | self isExtendedSelect
        ifTrue:
            [self
                selections:
                    (self selections
                    reject:
                        [:index |
                        index > objects size]) asOrderedCollection].
    ^self selections
        collect:
            [:index |
            objects at: index]! !

!CPColumnarListBox methods !   
selections
    "Answer the indeces of the selected items"

    self isMultiSelect | self isExtendedSelect
        ifFalse:
            [self selection isNil
                ifTrue: [^#()].
            ^Array with: self selection].
    ^selections asSortedCollection asArray! !

!CPColumnarListBox methods !  
fullyVisibleCells
    "Private - answer the number of cells that will display
     fully visible within the window "

    ^totalCells x @ super fullyVisibleCells y! !

!CPColumnarListBox methods !
highlightCell: aCell
	"Private - highlight a cell"

	| box highlightColor region |
	aCell isNil | self isValid not
		ifTrue: [^self].
	(self isCellVisible: aCell)
		ifFalse: [^self].
	(self isDisabled: aCell)
		ifTrue: [^self showFocusInCell: aCell].
	box := self rectForCell: aCell.
	highlightColor := UserLibrary getSysColor: ColorHighlight.
	self
		doGraphics:
			[region := self pen setClipRect: (self headerExtent rightBottom: self rectangle rightBottom).
			self
				pen fill: box color: highlightColor;
				foreColor: (UserLibrary getSysColor: ColorHighlighttext);
				setBackMode: Transparent.
			(aCell y between: 1 and: objects size) ifTrue: [
				self
					displayItem: (objects at: aCell y)
					inRect: box
					highlight: true ].
			self
				pen foreColor: self foreColor;
				setBackMode: Opaque.
			(self hasFocus and: [aCell = selectedCell])
				ifTrue:
					[self showFocusInCell: aCell "color: Color black".
					].
			(self inWindowBuilder not and: [self parent isDialogTopPane])
				ifTrue:
					[self
						pen foreColor: Color black;
						drawRectangle: self rect.
					].
			self pen destroyRegion: region.
			].! !

!CPColumnarListBox methods !
showFocusInCell: aCell
    "Private"

    self showFocusInCell: aCell color: Color black.! !

!CPColumnarListBox methods !   
keyboardInput: aKeyboardInputEvent
		"Private - keyboard input was received."
	| virtualKey newSel down |

	(selectedCell isNil or: [
		aKeyboardInputEvent isControlKeyDown
		| aKeyboardInputEvent isShiftKeyDown
	]) ifFalse: [
		virtualKey := aKeyboardInputEvent virtualKey.
		virtualKey = UpKey
			ifTrue:
				[self selection = 1
					ifTrue: [^self].
				down := false.
				newSel := (self selection - 1) max: 1.
				].
		virtualKey = DownKey
			ifTrue:
				[self selection = objects size
					ifTrue: [^self].
				newSel := (self selection + 1) min: objects size.
				down := true.
				].
		virtualKey = HomeKey
			ifTrue:
				[newSel := 1.
				down := true.
				].
		virtualKey = EndKey
			ifTrue:
				[down := false.
				newSel := objects size].
		virtualKey = PageUpKey
			ifTrue:
				[newSel := (selectedCell y - (self fullyVisibleCells y - 1) max: 1).
				down := newSel = 1.
				].
		virtualKey = PageDownKey
			ifTrue:
				[newSel := (selectedCell y + (self fullyVisibleCells y - 1) min: objects size).
				down := newSel ~= objects size.
				].
		virtualKey = LeftKey
			ifTrue:
				[self scrollHorizontal: self amountToScrollLeft.
				^self updateHorizontalSlider].
		virtualKey = RightKey
			ifTrue:
				[self scrollHorizontal: self amountToScrollRight.
				^self updateHorizontalSlider].
		down notNil ifTrue: [
			down
				ifTrue:
					[newSel := (newSel to: objects size)
						detect:
							[:i |
							(self isDisabled: 0 @ i) not]
						ifNone: [self selection].
					]
				ifFalse:
					[newSel := (newSel to: 1 by: -1)
						detect:
							[:i |
							(self isDisabled: 0 @ i) not]
						ifNone: [self selection].
					].
			(newSel notNil and: [(newSel > 0) & (newSel <= objects size)])
				ifTrue:
					[self isOkToChange = false
						ifFalse:
							[self selection: newSel.
							self isMultiSelect | self isExtendedSelect ifTrue: [
								self
									selections: (Array with: newSel);
									triggerMultiSelectEvent].
							self triggerSelectEvent.
							].
					].
			^self].
		].

	"Select something if nothing selected."
	(selectedCell isNil and: [objects size > 0
		and: [aKeyboardInputEvent virtualKey >= PageUpKey
		and: [aKeyboardInputEvent virtualKey <= DownKey]]])
		ifTrue: [
			newSel := 1.
			self selection: newSel.
			self isMultiSelect | self isExtendedSelect
				ifTrue: [
					self
						selections: (Array with: newSel);
						triggerMultiSelectEvent].
			self triggerSelectEvent.
			^self].

	^super keyboardInput: aKeyboardInputEvent! !

!CPColumnarListBox methods !
defaultShowLines
    "Private"

    ^false! !

!CPColumnarListBox methods !  
contents
    "Answer the object list passed in"

    ^objects! !

!CPColumnarListBox methods !   
showLines
    "Private"

    ^showLines! !

!CPColumnarListBox methods ! 
showHeader
    "Private"

    ^showHeader! !

!CPColumnarListBox methods !   
scrollTopCorner: newTopCorner
    "Private"

    self hasFocus
        ifTrue:
            [self hideFocus.
            ].
    super scrollTopCorner: newTopCorner.
    self hasFocus
        ifTrue:
            [self showFocus.
            ].! !

!CPColumnarListBox methods !   
columnResized
    "Return the column number of the column that was resized"

    ^self columnHeaders indexOf: (self fieldToResizeForPoint: self mouseLocation)! !

!CPColumnarListBox methods !  
hideFocus
    "Private"

    selectedCell isNil
        ifTrue: [^self].
    self showFocus.
    self displayLines.! !

!CPColumnarListBox methods !  
columnHeaders
    "Answer the list of column headers being displayed."

    ^self fields! !

!CPColumnarListBox methods !
showFocusInCell: aCell
color: ignored
    "Private"

    | y width |
    aCell isNil
        ifTrue: [^self].
    y := (self pointForCell: aCell) y.
    (self isCellVisible: aCell)
        ifFalse: [^self].
    width := self totalCells x * self cellSize x max: self rectangle width.
    self doGraphics: [self pen drawFocusRect: (topCorner x negated @ y rightBottom: width @ (y + self cellSize y))].! !

!CPColumnarListBox methods ! 
isCompatible: anObject
    "Is <anObject> compatible with the receiver"

	#obsoleteMethod.
    self columnHeaders
        do:
            [:column |
            (anObject respondsTo: column selector)
                ifFalse: [^false]].
    ^true! !

!CPColumnarListBox methods ! 
keepCellVisible: aCell
    "Private"

    | savedSelection answer |
    "Prevent the selection from drawing, since we'll be highlighting
     the item after scrolling"
    self isHandleOk ifFalse: [ ^false ].
    answer := false.
    aCell isNil
        ifFalse:
            [savedSelection := selectedCell.
            selectedCell := nil.
            answer := super keepCellVisible: aCell.
            selectedCell := savedSelection.
            ].
    ^answer! !

!CPColumnarListBox class methods !   
dragTargetDefaultEmphasis
		"Private - answer the type of target emphasis that should be employed
		if not otherwise specified."
	^'separator'! !

!CPColumnarListBox methods !  
open
    "Private"

	self triggerGetFieldsEvent.
    super open.! !

!CPColumnarListBox methods !   
disableMethod
    "Private"

    ^disableMethod! !

!CPColumnarListBox methods ! 
keepSelectionVisible
    "Private"

    ^super keepSelectionVisible
        ifTrue: [ self refreshSelection ].! !

!CPColumnarListBox methods ! 
font: aFont
	"Set the font of the listbox"

	| oldFont |
	oldFont := self font.
	super font: aFont.
	cellSize y: aFont height + 2.
	fields do: [:f |
		"If the field font is the same as the receiver's old font,
			then force the field font to be the new font."
		(f font wbEquals: oldFont)
			ifTrue: [f font: aFont]
			ifFalse: [f setFont: aFont]].
	self calcFieldSizes.! !

!CPColumnarListBox methods !
showLines: aBoolean
	"Set whether the vertical divider lines should be shown or not"

	showLines := aBoolean.
	self redraw.! !

!CPColumnarListBox methods !
dragTargetEraseSeparatorEmphasis
		"Private - erase the previously drawn separator emphasis."
	| lastIndex rect |
	lastIndex := self dragTargetSelection.
	lastIndex isNil ifTrue: [^self].
	rect := self rectForCell: 1 @ lastIndex.
	self
		dragTargetDrawEmphasisLineFrom: rect leftTop
		to: rect rightTop
! !

!CPColumnarListBox methods !   
rectForHeader: field
	"Private"

	| startX fieldIndex endX |
	startX := self rect left right: self leftMargin.
	1 to: (fieldIndex := fields indexOf: field) - 1 do: [:i |
		startX := startX right: (fields at: i) fieldPixelWidth + self fieldMargin].
	endX := startX right: field fieldPixelWidth + self fieldMargin.
	fieldIndex = 1 ifTrue: [startX := startX left: self leftMargin].
	^startX @ self rect top rightBottom:
		endX @ (self rect top down: headerHeight - 1)! !

!CPColumnarListBox methods ! 
dragScrolls
    "Private"

    ^true! !

!CPColumnarListBox methods !
resize: aRectangle
    "Private"

    rectangle := aRectangle.
    super resize: aRectangle.
    self setTopCell: 1 @ self getTopIndex.
    self reframe! !

!CPColumnarListBox methods ! 
rawSelections
    "Private"

    ^selections! !

!CPColumnarListBox methods !
fields: aFieldList
    "Set the fields for this listbox"

    fields := aFieldList.
    self calcFieldSizes.
    self isHandleOk
        ifTrue:
            [self invalidateRect: nil.
            self updateRectangle.
            ].! !

!CPColumnarListBox methods !  
updateTotalFieldWidth
    "Private"

    fields isNil
        ifTrue: [^self].
    totalFieldWidth := 0.
    fields
        do:
            [:field |
            totalFieldWidth := totalFieldWidth + self fieldMargin + field fieldPixelWidth].
    totalFieldWidth := totalFieldWidth + self fieldMargin.
    totalCells := totalFieldWidth - 1 // self cellSize x + 1 @ objects size.! !

!CPColumnarListBox methods !   
cursorOnFieldHeader: aPoint
    "Private"

    ^(self fieldToResizeForPoint: aPoint) notNil! !

!CPColumnarListBox methods ! 
fieldToReorderForPoint: aPoint
    "Private"

    | x offset |
    offset := topCorner x + aPoint x.
    x := 0.
    ^fields
        detect:
            [:field |
            offset between: x and: (x := x + field fieldPixelWidth + self fieldMargin)]
        ifNone: [nil].! !

!CPColumnarListBox methods !
triggerMultiSelectEvent

	self triggerSelectEvent.
    self triggerEvent: #multipleSelect: with: self selectedItems.! !

!CPColumnarListBox methods !
leftMargin
    "Private - answer the margin from the left side of the
     listbox, in pixels"

    ^4! !

!CPColumnarListBox methods ! 
reframe
    "Private - the window just resized"

	super reframe.
	"
	self isHandleOk ifTrue: [
		self
			calcFieldSizes;
			adjustAutoFields].
	"! !

!CPColumnarListBox methods ! 
validate

	super validate.
	self adjustAutoFields.! !

!CPColumnarListBox methods !  
singleSelect
    "Set the listbox to single select mode"

    self rawSelections: nil.
    self propertyAt: #cpStyle put: #singleSelect.! !

!CPColumnarListBox methods !   
scrollVertical: scrollAmount
    "Private - Scroll the pane down by anInteger
      number of pixels (if positive) or up by
      the absolute value of anInteger (if negative)."

    self scrollTopCorner: 0 @ scrollAmount.! !

!CPColumnarListBox methods !
refreshSelection
    "Private - Cause the selection to redraw"

    self refreshIndex: self selection! !

!CPColumnarListBox methods !   
wmHScroll: wordInteger
with: longInteger
    "Private - Process a WM_HSCROLL message."

    | type pos |
    type := wordInteger lowWord.
    type = SbLineup
        ifTrue: [self scrollHorizontal: self amountToScrollLeft].
    type = SbLinedown
        ifTrue: [self scrollHorizontal: self amountToScrollRight].
    type = SbPageup
        ifTrue: [self scrollHorizontal: self amountToPageLeft].
    type = SbPagedown
        ifTrue: [self scrollHorizontal: self amountToPageRight].
    (type = SbThumbposition) | (type = SbThumbtrack)
        ifTrue:
            [pos := wordInteger highWord.
            self updateHorizontalSliderTo: pos.
            self scrollHorizontal: self topCorner x - pos.
            ^nil].
    self updateHorizontalSlider.
    ^nil! !

!CPColumnarListBox methods !
button1DoubleClick: aPoint
    "Private"

    "selectedCell = (self boundedCellForPoint: aPoint)
        ifFalse: [self button1Down: aPoint].
    "
    (selectedCell notNil and: [
        (self rectForCell: selectedCell) containsPoint: aPoint])
            ifTrue: [super button1DoubleClick: aPoint].! !

!CPColumnarListBox methods !   
defaultCursor

    | aPoint |
    (self canResize 
        and: [(self isPointInHeader: (aPoint := self mouseLocation)) 
        and: [self cursorOnFieldBoundary: aPoint]])
        ifTrue: [^CursorManager sizeHorizontal].
    ^CursorManager arrow! !

!CPColumnarListBox methods !  
display
    "Private - respond to repaint messages"

    self
        doGraphics:
            [self
                pen fill: self scrollAreaRectangle color: self backColor;
                backColor: self backColor.
            self displayHeader.
            (objects isNil or: [objects isEmpty])
                ifFalse:
                    [self displayObjects.
                    self isMultiSelect | self isExtendedSelect
                        ifTrue:
                            [self rawSelections
                                do:
                                    [:i |
                                    self highlightCell: 1 @ i].
                            ]
                        ifFalse:
                            [self highlightCell: selectedCell.
                            ].
                    ].
            "self hasFocus
                ifTrue: [self showFocus]."
            self displayLines.
            (self inWindowBuilder not and: [self parent isDialogTopPane])
                ifTrue:
                    [self
                        pen foreColor: Color darkGray;
                        drawRectangle: self rect.
                    ].
            ].! !

!CPColumnarListBox methods ! 
colorMethod: aSymbol
    "Set the method used to access the color of each item"

    colorMethod := aSymbol! !

!CPColumnarListBox methods ! 
defaultDisableMethod
    "Private"

    ^nil! !

!CPColumnarListBox methods !
initialize
    "Private"

    super initialize.
    self singleSelect.
    cellSize := 50 @ self font height + 2.
    showHeader := self defaultShowHeader.
    fields := self defaultFields.
    showLines := self defaultShowLines.
    self contents: #().! !

!CPColumnarListBox methods ! 
redraw
    "Added by OSI. Redraw the entire window."
    #addedByOSI.
    self isHandleOk ifTrue: [
        self isVisible ifTrue: [
            self invalidateRect: nil ] ].! !

!CPColumnarListBox methods !
defaultHeaderColor
    "Private"

    ^Color white! !

!CPColumnarListBox methods !  
button1Down: aPoint
    "Private"

    | field |
    self doGraphics: [
        (self isPointInHeader: aPoint)
            ifTrue:
                [(self canResize and: [(field := self fieldToResizeForPoint: aPoint) notNil])
                    ifTrue: [self resizeField: field start: aPoint]
                    ifFalse:
                        [(self canReorder and: [(field := self fieldToReorderForPoint: aPoint) notNil])
                            ifTrue: [self reorderField: field start: aPoint]].
                ]
            ifFalse:
                [self isMultiSelect
                    ifTrue: [self button1DownMultiSelect: aPoint]
                    ifFalse: [
                        self isExtendedSelect
                            ifTrue: [self button1DownExtendedSelect: aPoint shift: false control: Notifier isControlKeyDown]
                            ifFalse: [super button1Down: aPoint]]]].! !

!CPColumnarListBox methods !
setTopCell: aCell
    "Private"

    | newTop |
    newTop := (0 @ (aCell y - 1)) * self cellSize.
    self scrollTopCorner: topCorner - newTop.
    self updateVerticalSlider.
    self updateHorizontalSlider.! !

!CPColumnarListBox methods !
headerExtent
    "Private - answer how big the header is"

    self showHeader
        ifTrue: [^0 @ headerHeight]
        ifFalse: [^0 @ 0].! !

!CPColumnarListBox methods ! 
selectedItem
	"Answer the selected item"

	selectedCell isNil ifTrue: [^nil].
	(selectedCell y between: 1 and: objects size) ifFalse: [^nil].
	^objects at: selectedCell y! !

!CPColumnarListBox methods !
pointForCell: aCell
    "Private"

    ^0 @ ((aCell y - 1) * self cellSize y) - topCorner + self headerExtent! !

!CPColumnarListBox methods !   
columnReordered
    "Return the column number of the column that was reordered"

    ^self columnClicked! !

!CPColumnarListBox methods !
fieldToResizeForPoint: aPoint
    "Private"

    | x offset |
    offset := topCorner x + aPoint x.
    x := self leftMargin - (self fieldMargin // 2).
    ^fields
        detect:
            [:field |
            x := x + field fieldPixelWidth + self fieldMargin.
            offset between: x - 2 and: x + 2]
        ifNone: [nil].! !

!CPColumnarListBox methods !   
columnHeaderClicked
    "Private"

    ^self fieldToReorderForPoint: self mouseLocation! !

!CPColumnarListBox methods ! 
contents: objectList
    "Set the contents of the listbox"

    objects := objectList.
    selectedCell := nil.
    self isMultiSelect | self isExtendedSelect
        ifTrue: [selections := Set new].
    self calcFieldSizes.
    selectedCell := nil.
    self isValid
        ifTrue:
            [self adjustAutoFields.
			self invalidateRect: nil.
            self updateRectangle.
            ].! !

!CPColumnarListBox methods ! 
rectForCell: cell
    "Private"

    | point |
    point := self pointForCell: cell.
    ^point rightBottom: self rectangle right @ (point y + self cellSize y)! !

!CPColumnarListBox methods !   
multipleSelect
    "Set the listbox to multiple select mode"

    self rawSelections: Set new.
    self propertyAt: #cpStyle put: #multipleSelect.! !

!CPColumnarListBox methods ! 
displayLines
	"Private"

	| x top bottom saveColor |
	self showLines ifFalse: [^self].
	x := topCorner x negated right: self leftMargin - 1.
	OperatingSystem isOS2 ifTrue: [x := x right: self rect left].
	top := self visibleRectangle top.
	bottom := self visibleRectangle bottom.
	saveColor := self pen foreColor.
	self pen foreColor: self wbDisplayContext borderColor.
	self doGraphics: [
		fields do: [:f |
			x := x right: f fieldPixelWidth + self fieldMargin.
			self pen lineFrom: x @ top to: x @ bottom]].
	self pen foreColor: saveColor.! !

!CPColumnarListBox methods ! 
reorderField: field
start: aPoint
    "Private"

    | currentPoint currentField tempField cursorLocation headerRect bitmap saveBitmap offset lastPoint |
    currentPoint := aPoint.
    CursorManager dragDropMove change.
    cursor := #leftRight.
    headerRect := self rectForHeader: field.
    headerRect := (headerRect left left: topCorner x) @ headerRect top
		extentFromLeftTop: headerRect extent.
    offset := currentPoint x - headerRect left.
    bitmap := self bitmapForHeader: field in: headerRect.
    saveBitmap := Bitmap screenExtent: headerRect extent.
    self pen fill: headerRect color: self headerColor.
    self
        whileMouseStillDownPrim: [:currentMousePosition |
            cursorLocation = self mouseLocation
                ifFalse:
                    [cursorLocation := self mouseLocation.
                    tempField := self fieldToReorderForPoint: cursorLocation.
                    tempField isNil
                        ifFalse: [currentField := tempField].
                    lastPoint isNil
                        ifFalse:
                            [self pen
                                    copyBitmap: saveBitmap
                                    from: bitmap boundingBox
                                    at: lastPoint @ 0
                                    rule: Srccopy.
                            ].
                    saveBitmap pen
                            copy: self pen
                            from: ((lastPoint := cursorLocation x - offset) @ 0 extent: headerRect extent)
                            at: 0 @ 0.
                    self pen
                            copyBitmap: bitmap
                            from: bitmap boundingBox
                            at: lastPoint @ 0
                            rule: Srccopy.
                    ].
            ].
    CursorManager normal change.
    (currentField isNil or: [field == currentField])
        ifTrue:
            [self
                displayHeader;
                displayLines.
            ^self].
    (fields indexOf: field) > (fields indexOf: currentField)
        ifTrue:
            [fields
                remove: field;
                add: field before: currentField]
        ifFalse:
            [fields
                remove: field;
                add: field after: currentField].
    bitmap release.
    saveBitmap release.
    self redraw.
    self updateRectangle.
    self event: #columnReordered. " OBSOLETE "
    self triggerEvent: #columnReordered: with: self columnReordered.! !

!CPColumnarListBox methods !   
selectedItems: aCollection
    "Set the selection of the listbox to aCollection."

    | indices index |
    indices := Set new.
    aCollection do: [ :obj |
        ( index := objects indexOf: obj ) > 0 ifTrue: [
            indices add: index ] ].
    self selections: indices.! !

!CPColumnarListBox methods !
unHighlightCell: aCell
    "Private"

    | box region |
    (aCell isNil | self isValid not or: [aCell y > objects size])
        ifTrue: [^self].
    (self isCellVisible: aCell)
        ifFalse: [^self].
    box := self rectForCell: aCell.
    region := self pen setClipRect: ((self rect leftTop rightAndDown: self headerExtent) rightBottom: self rect rightBottom).
    self
        doGraphics:
            [self
                pen fill: box color: self backColor;
                foreColor: self foreColor.
            self
                    displayItem: (objects at: aCell y)
                    inRect: box
                    highlight: false.
            (self inWindowBuilder not and: [self parent isDialogTopPane])
                ifTrue:
                    [self
                        pen foreColor: Color black;
                        drawRectangle: self rect.
                    ].
            self pen destroyRegion: region.
            ].! !

!CPColumnarListBox methods ! 
getTopIndex
    "Private - Answer the index of the object visible at the top of the
     listbox"

    ^self getTopCell y! !

!CPColumnarListBox methods !  
displayItem: anItem
inRect: aRect
highlight: aBoolean
	"Private - display a particular item, using aRect as the
	 rectangle in which to draw"

	| startX endX pen saveColor exposedRect |
	startX := aRect left right: self leftMargin.
	pen := self pen.
	saveColor := pen foreColor.
	exposedRect := self exposedRectangle.
	fields do: [:field |
		field setFont: self font.
		endX := startX right: field fieldPixelWidth + self fieldMargin.
		(startX isLeftOf: exposedRect right) & (endX isRightOf: exposedRect left) notNil ifTrue: [
			self wbDisplayContext
				foreColor: (
					(self isDisabled: anItem)
						ifTrue: [Color darkGray]
						ifFalse: [
							aBoolean
								ifTrue: [Color highlightText]
							ifFalse: [
								self colorMethod notNil
									ifTrue: [(anItem wbAttributeNamed: self colorMethod) ifNil: [self foreColor]]
									ifFalse: [self foreColor]]]);
				backColor: (
					aBoolean
						ifTrue: [Color highlightBackground]
						ifFalse: [self backColor]);
				cellBorder: (
					self showLines
						ifTrue: [#displayRightBorder]
						ifFalse: [#displayNoBorder]);
				cellInset: self class cellInset;
				cellBox: (startX @ aRect top rightBottom: endX @ aRect bottom).
			field
				wbDisplay: (
					field selector notNil
						ifTrue: [anItem wbAttributeNamed: field selector]
						ifFalse: [nil])
				with: self wbDisplayContext].
		startX := endX].
	pen foreColor: saveColor.! !

!CPColumnarListBox methods !  
colorMethod
    "Private"

    ^colorMethod! !

!CPColumnarListBox methods ! 
showFocus
    "Private"

    selectedCell isNil
        ifTrue: [^self].
    self showFocusInCell: selectedCell color: Color black! !

!CPColumnarListBox methods !
isDisabled: item
	"Answer whether the item is disabled"

	item isNil
		ifTrue: [^true].
	self disableMethod notNil
		ifTrue:
			[^item isPoint
				ifTrue:
					[item y > objects size
						ifTrue: [^true].
					(objects at: item y) wbAttributeNamed: self disableMethod]
				ifFalse:
					[item wbAttributeNamed: self disableMethod]].
	^false! !

!CPColumnarListBox methods !
isExtendedSelect
    "Answer whether the receiver is in extended selection mode."

    ^self cpStyle == #extendedSelect! !

!CPColumnarListBox methods ! 
selections: aCollectionOfIndices
    "Set the selection of the listbox to aCollectionOfIndices."

    | changed newSelections |
    self isMultiSelect | self isExtendedSelect ifFalse: [ ^self ].
    changed := false.
    self rawSelections do: [ :index |
        ( aCollectionOfIndices includes: index ) ifFalse: [
            self unHighlightCell: 1 @ index.
            changed := true ] ].
    newSelections := Set new.
    aCollectionOfIndices do: [ :index |
        ( index between: 1 and: objects size ) ifTrue: [
            newSelections add: index.
            ( self rawSelections includes: index ) ifFalse: [
                self highlightCell: 1 @ index.
                changed := true ] ] ].
    changed ifTrue: [
        self rawSelections: newSelections ].! !

!CPColumnarListBox methods ! 
bitmapForHeader: aField in: aRectangle
	"Answer the bitmap for the specified header."

	| answer inset |
	inset := self class headerInset.
	(fields indexOf: aField) = 1 ifTrue: [
		inset := (inset leftTop + (self leftMargin @ 0)) rightBottom: inset rightBottom].
	self doGraphics: [
		answer := Bitmap screenExtent: aRectangle extent.
		aField wbDisplayWith: (
			(self wbDisplayContextClass for: answer)
				scale: false;
				wrap: true;
				foreColor: self foreColor;
				backColor: self headerColor;
				cellBorder: #display3DBorder;
				cellInset: inset;
				yourself)].
	^answer! !

!CPColumnarListBox methods !  
displayHeader
	"Private - display the header of the listbox"

	| startX endX startY endY inset first rect |
	self showHeader ifFalse: [^self].
	rect := self visibleRectangle leftTop extentFromLeftTop:
		self visibleRectangle width @ self headerExtent y.
	self pen fill: rect color: self headerColor.
	inset := self class headerInset.
	self wbDisplayContext
		foreColor: self foreColor;
		backColor: self headerColor;
		cellBorder: #display3DBorder;
		clipRect: rect.
	startX := topCorner x negated.
	OperatingSystem isOS2 ifTrue: [startX := startX right: self rect left].
	startY := self visibleRectangle top.
	endY := startY down: self headerExtent y.
	first := true.
	fields do: [:f |
		endX := startX right: f fieldPixelWidth + self fieldMargin.
		first
			ifTrue: [
				endX := endX right: self leftMargin.
				self wbDisplayContext cellInset: (
					inset leftTop + (self leftMargin @ 0) rightBottom: inset rightBottom).
				first := false]
			ifFalse: [
				self wbDisplayContext cellInset: inset].
		self wbDisplayContext cellBox: ((startX @ startY leftAndUp: 1) rightBottom: endX @ endY).
		f wbDisplayWith: self wbDisplayContext.
		startX := endX].
	startX := self visibleRectangle left.
	endX := self visibleRectangle right.
	startY := endY up: Rectangle leftBottomUnit y.
	endY := startY.
	self pen
		foreColor: Color black;
		lineFrom: startX @ startY to: endX @ endY.
	self wbDisplayContext
		clipRect: nil.! !

!CPColumnarListBox methods !   
button1DownExtendedSelect: aPoint shift: shiftKey control: controlKey
    "Private"

    | cell newSelection currentSelection originalSelection anchor first |
    (self subpaneButton1Down: aPoint) ifFalse: [^self].
    anchor := self boundedCellForPoint: aPoint.
    (self inBounds: anchor) ifFalse: [^self].
    (self isDisabled: anchor) ifTrue: [^self].
    self doSetFocus.

    self isOkToChange == false ifTrue: [
        self whileMouseStillDown: [:currentMousePosition | ].
        ^self].

    currentSelection := self rawSelections.
    currentSelection isNil ifTrue: [currentSelection := Set new].
    shiftKey & currentSelection notEmpty ifTrue: [
        "Get an initial value from a set."
        currentSelection do: [:index | cell := index].
        cell := currentSelection inject: cell into: [:a :b | a min: b].
        cell <= anchor y ifFalse: [
            cell := currentSelection inject: cell into: [:a :b | a max: b]].
        anchor := anchor x @ cell.
        cell := self boundedCellForPoint: aPoint].
    originalSelection := currentSelection copy.

    self whileMouseStillDown: [:mousePoint |
        cell := self boundedCellForPoint: mousePoint.
        self dragScrolls ifTrue: [self keepCellVisible: cell].
        controlKey
            ifTrue: [
                newSelection := originalSelection copy.
                (cell y min: anchor y) to: (cell y max: anchor y) do: [:index |
                    (newSelection includes: index)
                        ifTrue: [newSelection remove: index]
                        ifFalse: [newSelection add: index]]]
            ifFalse: [
                newSelection := (cell y min: anchor y) to: (cell y max: anchor y)].
        newSelection copy do: [:index |
            (self isDisabled: 1 @ index) ifTrue: [
                newSelection class == Set ifFalse: [newSelection := newSelection asSet].
                newSelection remove: index]].
        currentSelection do: [:index |
            (newSelection includes: index) ifFalse: [
                self unHighlightCell: 1 @ index]].
        newSelection do: [:index |
            (currentSelection includes: index) ifFalse: [
                self highlightCell: 1 @ index]].
        currentSelection := newSelection.
        first := false].
    self rawSelections: currentSelection asSet.
    selectedCell := cell.
    self showFocusInCell: selectedCell.
    self triggerMultiSelectEvent.! !

!CPColumnarListBox methods !
dragSourceNeedsObjectDefault: dragSession
        "Private - provide default for requested drag items if no handler
        is provided by supplying the currently selected item."
    | indices dragObjects |

	indices := self selections reject: [ :index | index > objects size ].
	indices isEmpty ifTrue: [ ^self ].
	self dragSourceSelection: indices.

    dragObjects := self selectedItems asArray collect: [ :each |
		dragSession objectClass new
			object: each;
			string: each asString;
			yourself ].
    dragSession objects: dragObjects.! !

!CPColumnarListBox methods !   
columnClicked
    "Return the column number of the column that was clicked on"

    ^self columnHeaders indexOf: self columnHeaderClicked! !

!CPColumnarListBox methods !   
showHeader: aBoolean
    "Set whether the header should be shown or not"

    showHeader := aBoolean! !

!CPColumnarListBox methods !
disableMethod: aSymbol
    "Set the method used to determine whether an item is disabled"

    disableMethod := aSymbol! !

!CPColumnarListBox methods ! 
defaultDragDropCursor
    "Private - answer the cursor to display for this
     drag-drop operation, since no handler was
     registered to provide one."

    #obsoleteMethod.
    ^self selections size > 1
        ifTrue:
            [Notifier isAltKeyDown
                ifTrue: [CursorManager dragDropMove]
                ifFalse: [CursorManager dragDropCopy]]
        ifFalse:
            [Notifier isAltKeyDown
                ifTrue: [CursorManager dragDropMove]
                ifFalse: [CursorManager dragDropCopy]]! !

!CPColumnarListBox methods !
isCellVisible: aCell
    "Private"

    | topCell bottomCell |
    topCell := self getTopCell y.
    aCell y < topCell
        ifTrue: [^false].
    bottomCell := topCell + self fullyVisibleCells y.
    aCell y > bottomCell
        ifTrue: [^false].
    ^true! !

!CPColumnarListBox methods !  
setTopIndex: anIndex
    "Scroll listbox so that the item at anIndex is at the top of
     the listbox"

    self setTopCell: 1 @ anIndex! !

!CPColumnarListBox methods !  
minimumFieldWidth
    "Private"

    ^self fieldMargin * 2! !

!CPColumnarListBox methods !  
triggerGetFieldsEvent

    self event: #getFields. " OBSOLETE "
    self triggerEvent: #getFields.! !

!CPColumnarListBox methods !  
selection
    "Answer the index of the selected item"

    selectedCell isNil
        ifTrue: [^nil].
    ^selectedCell y! !

!CPColumnarListBox methods ! 
dragTargetDrawSeparatorEmphasis: aPoint
        "Private - draw the target emphasis for the receiver
        given the cursor location aPoint (relative to the receiver).
        Emphasis is drawn indicating spaces between existing
        items as individual targets."

    | index lastIndex rect |
    index := (self boundedCellForPoint: aPoint) y.
        "only update if changed"
    lastIndex := self dragTargetSelection.
    index = lastIndex ifTrue: [ ^nil ].  "don't draw emphasis if already drawn."
    lastIndex notNil ifTrue: [ self dragTargetEraseSeparatorEmphasis ].
    rect := self rectForCell: 1 @ index.
    self
        dragTargetDrawEmphasisLineFrom: rect leftTop
        to: rect rightTop.
    self dragTargetSelection: index.
! !

!CPColumnarListBox methods !
selectedIndex
    "Answer the index of the selected item"

    ^self selection! !

!CPColumnarListBox methods !  
defaultDragDropObjects
    "Answer the list of items that should be
     passed during drag-drop"

    | dragObjects |
    #obsoleteMethod.
    (dragObjects := self getDefaultDragObjects) notNil
        ifTrue: [^dragObjects].
    dragObjects := self selectedItems asArray.
    dragObjects isEmpty
        ifTrue: [^nil].
    self renderAsObjects
        ifFalse:
            [dragObjects := dragObjects
                collect:
                    [:each |
                    each asString]].
    ^dragObjects! !

!CPColumnarListBox methods !   
defaultFields
    "Private - set up the fields for a blank listbox"

    ^OrderedCollection
            with: (CPCLBField new header: 'Field 1')
            with: (CPCLBField new header: 'Field 2')
            with: (CPCLBField new header: 'Field 3').! !

!CPColumnarListBox methods !  
contentsNoResize: objectList
    "Set the contents of the listbox"

    objects := objectList.
    selectedCell := nil.
    self isMultiSelect | self isExtendedSelect
        ifTrue: [selections := Set new].
    selectedCell := nil.
    self isValid
        ifTrue: [
            self invalidateRect: nil.
            self updateRectangle].! !

!CPColumnarListBox methods !
adjustAutoFields
	"Private - Adjust the auto width field sizes."

	| dynamicFields extra w f |
	objects isNil | fields isNil ifTrue: [^self].
	dynamicFields := self columnHeaders select: [:each | each autoFieldWidth].
	extra := self scrollAreaRectangle width - totalFieldWidth + 6.
	extra > 0 & dynamicFields notEmpty ifTrue: [
		totalFieldWidth := totalFieldWidth + extra.
		1 to: dynamicFields size do: [:index |
			f := dynamicFields at: index.
			w := extra // (dynamicFields size - index + 1).
			extra := extra - w.
			f fieldPixelWidth: f fieldPixelWidth + w]].! !

!CPColumnarListBox methods !  
isSelected: index
    "Answer whether the item is selected"

    self isMultiSelect | self isExtendedSelect
        ifTrue: [^self rawSelections includes: index].
    selectedCell isNil ifTrue: [^false].
    ^index == selectedCell y! !

!CPColumnarListBox methods ! 
refreshIndex: anIndex
    "Private - Redisplay the object at index anIndex"

    (selectedCell notNil and: [anIndex = selectedCell y])
        ifTrue:
            [self highlightCell: 1 @ anIndex.
            ]
        ifFalse:
            [anIndex notNil
                ifTrue:
                    [self unHighlightCell: 1 @ anIndex.
                    ]].! !
   
UndefinedObject removeSelector: #osiIgnoredInitializerSource!
   
"======== Initializers"!