

!Collection methodsFor: 'as yet unclassified' !
 
isCollection

    ^true! !

!ProcessScheduler methodsFor: 'as yet unclassified' !
   
resume: aProcess
        "Add aProcess to the process scheduler's
         queue of ready processes.  If aProcess
         has the highest priority, make it the
         current process."
    | priority |
    Process enableInterrupts: false.
	aProcess runable == 0 "If the process is not runable, ignore the resume"
		ifTrue: [  ^ Process enableInterrupts: true ].
    priority := aProcess priority.
    (readyProcesses at: priority)
        addLast: aProcess.
    CurrentProcess priority < priority
        ifTrue: [
            Processor currentProcessIsRecursive
                ifTrue: [HaveBeenSignals:= true.
                             Process enableInterrupts: true]
                ifFalse: [Processor yield]]
        ifFalse: [Process enableInterrupts: true]! !

!Object methodsFor: 'as yet unclassified' !
   
isCollection

    ^false! !

!Object methodsFor: 'as yet unclassified' !

hash: upperBound 
    "operation on CORBA psuedo Object to hash an object with an upper 
    maximum bound (we ignore the bound)"

    ^self hash! !

!Object methodsFor: 'as yet unclassified' !
 
asString
    ^self printString! !

!Float methodsFor: 'dst' !

reverseByteOrder
	"Swap byte order when booting up on a different-endian
	machine than the image was saved on."

	| bytes |

	bytes := self basicSize.
	1 to: bytes by: 2 do:
		[:i |	| t |
		t := self basicAt: i.
		self basicAt: i put: (self basicAt: bytes-i+1).
		self basicAt: bytes-i+1 put: t]! !

!ByteArray methodsFor: 'dst' !
 
byteAt: index 
	"Answer the byte at the given index."

	^self basicAt: index! !

!ByteArray methodsFor: 'dst' !

byteAt: index put: aByte 
	"Store a byte at the given index."

	^self basicAt: index put: aByte! !

!Process methodsFor: 'dst' !
   
suspend
"patch AFC"! !

!False methodsFor: 'dst' !
   
asValue
	^SharedValue new value: self; yourself! !

!Dictionary methodsFor: 'refactoring' !
  
associations
	"Answer an OrderedCollection containing the receiver's associations in an 
	arbitrary order."

	| aCollection |
	aCollection := OrderedCollection new: self size.
	self associationsDo: [:assn | aCollection add: assn].
	^aCollection! !

!Collection methodsFor: 'dst' !
   
growToAtLeast: anInteger
	"The receiver becomes at least as large as anInteger."
	
	"This is not a copy of the receiver, so all shared references survive."

	anInteger <= self size
		ifTrue: [^self].
	self growTo: anInteger! !

!Character methodsFor: 'dst' !
 
lf

	^Lf! !

!Character class methodsFor: 'dst' !
   
lf

	^Lf! !

!Object methodsFor: 'dst' !

setAccess: aCollection user: aUserId 
	"set the access rights for a user"

	| acl accessRights |
	(self respondsTo: #accessControlList)
		ifTrue: [((aCollection isKindOf: Collection)
				and: [(aUserId isKindOf: Symbol)
						or: [aUserId isKindOf: NCSuuid]])
				ifTrue: 
					[((acl := self accessControlList) isKindOf: IdentityDictionary)
						ifFalse: [^(CORBAConstants at: #'::Security::NotSupported')
								raiseWith: Dictionary new errorString: ' incorrect implementation'].
					accessRights := IdentitySet new.
					aCollection do: [:sym | accessRights add: sym].
					acl at: aUserId put: accessRights.
					self changed: #accessControlList]
				ifFalse: [self class badParamError: 0]]
		ifFalse: [^(CORBAConstants at: #'::Security::NotSupported')
				raiseWith: Dictionary new errorString: ' this object does not support access control']! !

!Array methodsFor: 'dst' !
  
growToAtLeast: anInteger
	"The receiver becomes at least as large as anInteger."
	
	"This is not a copy of the receiver, so all shared references survive."

	anInteger <= self size
		ifTrue: [^self].
	self changeSizeTo: anInteger + self growSize! !

!Array methodsFor: 'dst' !
   
changeSizeTo: newSize
	"The receiver changes size."
	
	"This is not a copy of the receiver, so all shared references survive."

	| newArray |
	newArray := self copyEmpty: newSize.
	newArray
		replaceFrom: 1
		to: (self size min: newSize)
		with: self.
	self become: newArray! !

!SocketAccessor methodsFor: 'refactoring' !
 
releaseHandle

	handle := nil! !

!Exception methodsFor: 'dst' !

parameter

	^self class name! !

!Collection methodsFor: 'dst' !

copyEmpty: aSize
	"Answer a copy of the receiver that contains no elements.

	This method should be redefined in subclasses that add
	instance variables, so that the state of those variables
	is preserved"

	^self class new: aSize! !

!ByteArray methodsFor: 'dst' !
  
replaceBytesFrom: byteStart
    to: byteStop
    with: aByteObject
    startingAt: byteRepStart
        "Private - replace the bytes of the receiver at index
         positions start through stop with consecutive
         bytes of aByteObject beginning at index
         position repStart.  Answer the receiver."
    | index1 index2 |
    <primitive: 80>
    aByteObject class isBytes
        ifFalse: [self error: 'source must be a byte object'].
    (self == aByteObject and: [byteRepStart < byteStart])
        ifTrue: [ "do backward move for same object"
            index2 := byteRepStart + byteStop - byteStart.
            index1 := byteStop.
            [byteStart <= index1]
                whileTrue: [
                self
                    at: index1
                    put: (aByteObject basicAt: index2) asInteger.
                index1 := index1 - 1.
                index2 := index2 - 1].
            ^self].
    "do forward move"
    index2 := byteRepStart.
    index1 := byteStart.
    [index1 <= byteStop]
        whileTrue: [
            self
                at: index1
                put: (aByteObject basicAt: index2) asInteger.
            index1 := index1 + 1.
            index2 := index2 + 1]! !

!ByteArray methodsFor: 'dst' !

asStringEncoding: encoding
	"Convert self encoded in encoding to Unicode string."

	| map cache |
	cache := LastDecodeMap. 
	cache key == encoding
		ifTrue: [map := cache value]
		ifFalse:
			[map := Lock critical:
					[Decodings at: encoding ifAbsent: [#createMap]].
			map == #createMap
				ifTrue: [map := self class createDecodeMapFor: encoding].
			map == nil
				ifFalse: [LastDecodeMap := encoding -> map]].
	map == #identityMap
		ifTrue: [^self copy changeClassToThatOf: ''].
	^map == nil
		ifTrue: [super asStringEncoding: encoding]
		ifFalse: [self asStringFromMap: map encoding: encoding]! !

!Exception methodsFor: 'dst' !
 
reject
	"The handler has decided it doesn't want to handle this Signal.  Continue searching for
	another handler."

	^self pass.! !

!Object methodsFor: 'as yet unclassified' !
  
-> anObject
        "Answer a new instance of an Association
        with key the receiver and value anObject."

    "** Courtesy OTI (added)"

	#addedByOSI.
    ^Association key: self value: anObject! !